@@ -1,5 +1,5 @@
 import numpy as np
-from spider.vehicle_model.bicycle import Bicycle
+from spider.control.vehicle_model import Bicycle
 
 
 # 考虑一下要不要改成 traj = [pose]
@@ -32,6 +32,24 @@ class Path:
     def densify(self):
         pass
 
+    def _list(self, seq):
+        return seq.tolist() if hasattr(seq, "tolist") else list(seq)
+
+    def to_dict(self, frenet=False):
+        d = {
+            "type": self.__class__.__name__,
+            'x': self._list(self.x),
+            'y': self._list(self.y),
+            'yaw': self._list(self.heading),
+            'curvature': self._list(self.curvature),
+        }
+        if frenet:
+            d.update({
+                's': self._list(self.s),
+                'l': self._list(self.l),
+            })
+        return d
+
 class Trajectory(Path):
 
     _sequential_properties = [
@@ -181,6 +199,20 @@ class Trajectory(Path):
         self.l = []
         self.t = []
 
+    def to_dict(self, frenet_path=False):
+         d = super(Trajectory, self).to_dict(frenet_path)
+         d.update({
+             "type": self.__class__.__name__,
+             't': self._list(self.t),
+             'v': self._list(self.v),
+             'a': self._list(self.a),
+             'steer': self._list(self.steer),
+             'steer_velocity': self._list(self.steer_velocity),
+             'jerk': self._list(self.jerk),
+             'centripetal_acceleration': self._list(self.centripetal_acceleration),
+         })
+         return d
+
 
 
 class FrenetTrajectory(Trajectory):
@@ -210,6 +242,27 @@ class FrenetTrajectory(Trajectory):
     def trajectory_sl_array(self):
         return np.column_stack((self.s, self.l))
 
+    def to_dict(self):
+        d = super(FrenetTrajectory, self).to_dict(frenet_path=True)
+        d.update({
+            "type": self.__class__.__name__,
+
+            's': self._list(self.s),
+            's_dot': self._list(self.s_dot),
+            's_2dot': self._list(self.s_2dot),
+            's_3dot': self._list(self.s_3dot),
+
+            'l': self._list(self.l),
+            'l_dot': self._list(self.l_dot),
+            'l_2dot': self._list(self.l_2dot),
+            'l_3dot': self._list(self.l_3dot),
+
+            'l_prime': self._list(self.l_prime),
+            'l_2prime': self._list(self.l_2prime),
+            'l_3prime': self._list(self.l_3prime),
+        })
+        return d
+
 
 if __name__ == '__main__':
     a = Trajectory(50)
