@@ -1,30 +1,22 @@
 import math
-from typing import List
+from typing import Iterable
 import time
-import numpy as np
 import warnings
 
 import spider
+import numpy as np
 from spider.planner_zoo.BasePlanner import BasePlanner
 
-from spider.elements.map import RoutedLocalMap
-from spider.elements.trajectory import FrenetTrajectory
-from spider.elements.vehicle import VehicleState
-from spider.elements.Box import TrackingBoxList, TrackingBox
+from spider.elements import RoutedLocalMap,FrenetTrajectory,VehicleState,TrackingBoxList, TrackingBox
 
-from spider.sampler.PolynomialSampler import QuinticPolyminalSampler, QuarticPolyminalSampler
-from spider.sampler.Combiner import LatLonCombiner
+from spider.sampler.LatticeSampler import LatticeSampler
 from spider.evaluator import FrenetCostEvaluator
 
-from spider.utils.transform.frenet import FrenetCoordinateTransformer
+from spider.utils.transform.frenet import FrenetTransformer
 from spider.utils.collision import BoxCollisionChecker
 
 from spider.constraints import CartConstriantChecker
 
-# from concurrent.futures import ProcessPoolExecutor
-# def transform_trajectory(t, coordinate_transformer):
-#     return coordinate_transformer.frenet2cart4traj(t, order=2)
-
 class LatticePlanner(BasePlanner):
     def __init__(self, config=None):
         super(LatticePlanner, self).__init__(config)
@@ -34,13 +26,13 @@ class LatticePlanner(BasePlanner):
         #     self.config.update(config)
 
         self.local_map = RoutedLocalMap()
-        self.coordinate_transformer = FrenetCoordinateTransformer() # 要维护几个坐标系呢？
-        # self.predictor = None
-        self.longitudinal_sampler = QuarticPolyminalSampler(self.config["end_T_candidates"],
-                                                            self.config["end_v_candidates"])
-        self.lateral_sampler = QuinticPolyminalSampler(self.config["end_s_candidates"],
-                                                       self.config["end_l_candidates"])
-        self.trajectory_combiner = LatLonCombiner(self.config["steps"], self.config["dt"]) # 默认路径-速度解耦的重新耦合
+        self.coordinate_transformer = FrenetTransformer() # 要维护几个坐标系呢？
+
+        self.trajectory_sampler = LatticeSampler(
+            self.steps, self.dt,
+            self.config["end_T_candidates"], self.config["end_v_candidates"],
+            self.config["end_s_candidates"], self.config["end_l_candidates"]
+        )
         self.trajectory_evaluator = FrenetCostEvaluator()
 
         # self.collision_checker = BoxCollisionChecker(self.config["ego_veh_length"], self.config["ego_veh_width"])
@@ -91,20 +83,12 @@ class LatticePlanner(BasePlanner):
         return self._candidate_trajectories, self._candidate_trajectories_cost
 
 
-    def constraint_check(self, sorted_candidate_trajectories:List[FrenetTrajectory], sorted_cost, obstacles:TrackingBoxList):
+    def constraint_check(self, sorted_candidate_trajectories:Iterable[FrenetTrajectory], sorted_cost, obstacles:TrackingBoxList):
         for traj, cost in zip(sorted_candidate_trajectories, sorted_cost):
-            traj = self.coordinate_transformer.frenet2cart4traj(traj, order=2) # 在笛卡尔坐标系下做碰撞检测
+            # traj = self.coordinate_transformer.frenet2cart4traj(traj, order=2) # 在笛卡尔坐标系下做碰撞检测
             if self.constraint_checker.check(traj, obstacles):
                 return traj, cost
 
-            # if np.any(np.array(traj.v) > self.config["max_speed"]): continue
-            # if np.any(np.array(traj.v) < self.config["min_speed"]): continue
-            # if np.any(np.array(traj.a) > self.config["max_acceleration"]): continue
-            # if np.any(np.array(traj.a) < -self.config["max_deceleration"]): continue
-            # if np.any(np.abs(traj.curvature) > self.config["max_curvature"]): continue
-            # if self.collision_checker.check_trajectory(traj, obstacles): continue
-            # return traj, cost
-
 
         return None, 0
 
@@ -113,26 +97,10 @@ class LatticePlanner(BasePlanner):
 
     def build_frenet_lane(self, target_lane_idx):
         if target_lane_idx < 0 or target_lane_idx >= len(self.local_map.lanes):
-            raise ValueError("Invalid target lane index")
+            raise ValueError("Invalid target lane record_index")
         target_lane = self.local_map.lanes[target_lane_idx]
         self.coordinate_transformer.set_reference_line(target_lane.centerline, target_lane.centerline_csp)
 
-    # def match_lanes(self, ego_veh_state:VehicleState):
-    #     # 已经放在map的函数里了
-    #     if len(self.local_map.lanes) == 0:
-    #         raise ValueError("No lanes!")
-    #
-    #     x,y = ego_veh_state.x(), ego_veh_state.y()
-    #     min_idx, min_dist = -1, math.inf
-    #     for idx in range(len(self.local_map.lanes)):
-    #         self.build_frenet_lane(idx)
-    #         fstate = self.coordinate_transformer.cart2frenet(x, y, order=0)
-    #         dist = math.fabs(fstate.l)
-    #         if dist<min_dist:
-    #             min_idx, min_dist = idx, dist
-    #     return min_idx
-
-
     def plan(self, ego_veh_state:VehicleState, obstacles:TrackingBoxList, local_map:RoutedLocalMap=None) -> FrenetTrajectory:
         """
         输入定位、物体、（地图optional,更新频率比较慢。建议在外面单独写set地图的逻辑）
@@ -157,16 +125,13 @@ class LatticePlanner(BasePlanner):
         predicted_obstacles = obstacles.predict(self.config["dt"] * np.arange(self.config["steps"]))
 
         # 轨迹采样
-        long_samples = self.longitudinal_sampler.sample((fstate_start.s, fstate_start.s_dot, fstate_start.s_2dot))
-        lat_samples = self.lateral_sampler.sample((fstate_start.l, fstate_start.l_prime, fstate_start.l_2prime))
-        candidate_trajectories = self.trajectory_combiner.combine(lat_samples, long_samples) # todo:这一步特别耗时!!
+        candidate_trajectories = self.trajectory_sampler.sample(
+            (fstate_start.s, fstate_start.s_dot, fstate_start.s_2dot),
+            (fstate_start.l, fstate_start.l_prime, fstate_start.l_2prime)
+        )
 
 
         # 轨迹坐标转换，把每个轨迹点转到笛卡尔坐标
-        # todo: qzl:这一步耗时非常严重，有2个建议：
-        #  1. 评估暂时用不到笛卡尔坐标，碰撞检测目前在笛卡尔坐标下，所以可以在碰撞检测的时候再转换，不用提前把所有的都做坐标转换
-        #  2. 把碰撞检测直接整个放在Frenet坐标下，即先把障碍物及其预测轨迹变换到Frenet坐标，这样子所有的环节都在Frenet坐标进行，最后输出前再转换到笛卡尔
-        #  p.s.第一点的补充：坐标转换可以暂时先存储成函数或生成器，暂时先不执行，在碰撞检测的时候才执行
         # candidate_trajectories = [self.coordinate_transformer.frenet2cart4traj(t, order=2) for t in candidate_trajectories]
         # 并行计算
         # with ProcessPoolExecutor() as executor:
@@ -175,6 +140,9 @@ class LatticePlanner(BasePlanner):
 
         # 评估+筛选
         sorted_candidates, sorted_cost = self.trajectory_evaluator.evaluate_candidates(candidate_trajectories)
+
+        sorted_candidates = (self.coordinate_transformer.frenet2cart4traj(t, order=2) for t in sorted_candidates)
+        # 由于frenet2cart4traj,order=2计算量很大，所以使用了生成器表达式，提高初始化时候的效率
         optimal_trajectory, min_cost = self.constraint_check(sorted_candidates, sorted_cost, predicted_obstacles)
 
         self._candidate_trajectories, self._candidate_trajectories_cost = sorted_candidates, sorted_cost
@@ -192,13 +160,32 @@ class LatticePlanner(BasePlanner):
 
         return optimal_trajectory
 
+    # def match_lanes(self, ego_veh_state:VehicleState):
+    #     # 已经放在map的函数里了
+    #     if len(self.local_map.lanes) == 0:
+    #         raise ValueError("No lanes!")
+    #
+    #     x,y = ego_veh_state.x(), ego_veh_state.y()
+    #     min_idx, min_dist = -1, math.inf
+    #     for idx in range(len(self.local_map.lanes)):
+    #         self.build_frenet_lane(idx)
+    #         fstate = self.coordinate_transformer.cart2frenet(x, y, order=0)
+    #         dist = math.fabs(fstate.l)
+    #         if dist<min_dist:
+    #             min_idx, min_dist = idx, dist
+    #     return min_idx
+
 
 if __name__ == '__main__':
     from spider.elements.map import Lane
     from spider.elements.vehicle import *
-    from spider.elements.Box import obb2vertices
+    from spider.elements.box import obb2vertices
     import matplotlib.pyplot as plt
-    import cv2
+    import spider
+    try:
+        import cv2
+    except (ModuleNotFoundError, ImportError) as e:
+        cv2 = spider._virtual_import("cv2", e)
 
 
     def multilane():
