@@ -6,6 +6,12 @@ import numpy as np
 import pickle
 import io
 
+try:
+    import cv2
+except (ModuleNotFoundError, ImportError) as e:
+    import spider
+    cv2 = spider._virtual_import("cv2", e)
+
 
 # plt.gca().set_facecolor('white')
 #     plt.gca().set_xticks([])
@@ -68,19 +74,17 @@ class SnapShot:
         self.record_video = False
         self.video_writer = None
         if record_video:
-            import cv2
             self.record_video = True
             # todo: 下面的内容换成可以由video_kwargs设置，设默认值可以先设一个字典，然后update一下
             # video_path =
             fourcc = cv2.VideoWriter_fourcc(*'XVID')
-            fps = 10
+            fps = 10 if "fps" not in video_kwargs else video_kwargs["fps"]
             self.video_settings = (video_path, fourcc, fps)
 
             # self.video_writer = cv2.VideoWriter(self.video_path, fourcc, fps, (640, 480))
 
     def _build_video_writer(self, image_example):
         height, width = image_example.shape[:2]
-        import cv2
         self.video_writer = cv2.VideoWriter(*self.video_settings, (width, height))
         return self.video_writer
 
@@ -92,7 +96,10 @@ class SnapShot:
     def album_size(self):
         return len(self.album)
 
-    def snap(self, ax:plt.Axes):
+    def snap(self, ax:plt.Axes=None):
+        if ax is None:
+            ax = plt.gca()
+
         if self._snap_trigger() or self.record_video:
             img = np.frombuffer(ax.figure.canvas.buffer_rgba(), dtype=np.uint8)
             try: # todo:这边搞不清楚怎么回事，有时候上面work有时候下面work，暂时先这样子写吧
@@ -148,6 +155,13 @@ class SnapShot:
                 no_tick(ax)
         return
 
+    def release_writer(self):
+        if self.video_writer is not None:
+            self.video_writer.release()
+
+    def close(self):
+        plt.close('all') # 不应该是all
+        self.release_writer()
 
     def save(self):
         '''
